from ._thanker import Thanker


__version__ = "0.0.3"
__url__ = "https://github.com/WardPearce/thanker"
__description__ = "Don't be a wanker, be a thanker!"
__author__ = "WardPearce"
__author_email__ = "wardpearce@protonmail.com"
__license__ = " Apache-2.0"


__all__ = [
    "Thanker"
]
